# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

REQUEST_TIMEOUT = 60
REQUEST_RETRIES = 3
BACK_OFF_TIME = 2

FEED_API_PROD_URL = 'https://api.ebay.com/buy/feed/v1_beta/'
FEED_API_SANDBOX_URL = 'https://api.sandbox.ebay.com/buy/feed/v1_beta/'

# max content that can be downloaded in one request, in bytes
PROD_CHUNK_SIZE = 104857600
SANDBOX_CHUNK_SIZE = 10485760

TOKEN_BEARER_PREFIX = 'Bearer '

AUTHORIZATION_HEADER = 'Authorization'
MARKETPLACE_HEADER = 'X-EBAY-C-MARKETPLACE-ID'
CONTENT_TYPE_HEADER = 'Content-type'
ACCEPT_HEADER = 'Accept'
RANGE_HEADER = 'Range'

CONTENT_RANGE_HEADER = 'Content-Range'

RANGE_PREFIX = 'bytes='

APPLICATION_JSON = 'application/json'

QUERY_SCOPE = 'feed_scope'
QUERY_CATEGORY_ID = 'category_id'
QUERY_SNAPSHOT_DATE = 'snapshot_date'
QUERY_DATE = 'date'


SUCCESS_CODE = 0
FAILURE_CODE = -1

SUCCESS_STR = 'Success'
FAILURE_STR = 'Failure'

DATA_FRAME_CHUNK_SIZE = 2*(10**4)  # rows



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import sys
import pandas as pd
from distutils.util import strtobool


def convert_to_bool_false(data):
    try:
        bool_value = strtobool(data)
        return pd.np.bool(bool_value)
    except (ValueError, TypeError, AttributeError):
        return pd.np.bool(False)


def convert_to_float_max_int(data):
    try:
        return pd.np.float(data)
    except (ValueError, TypeError, AttributeError):
        return pd.np.float(sys.maxsize)


def convert_to_float_zero(data):
    try:
        return pd.np.float(data)
    except (ValueError, TypeError, AttributeError):
        return pd.np.float(0)


def get_inclusive_less_query(column_name, upper_limit):
    if not upper_limit:
        return ''
    return '%s <= %s' % (column_name, upper_limit)


def get_inclusive_greater_query(column_name, lower_limit):
    if not lower_limit:
        return ''
    return '%s >= %s' % (column_name, lower_limit)


def get_list_number_element_query(column_name, value_list):
    if not value_list:
        return ''
    list_str = ','.join(str(element) for element in value_list)
    return '%s IN (%s)' % (column_name, list_str)


def get_list_string_element_query(column_name, value_list):
    if not value_list:
        return ''
    list_str = (','.join('\'' + item + '\'' for item in value_list))
    return '%s IN (%s)' % (column_name, list_str)



import unittest
from os import remove
from os.path import isfile, getsize, split, abspath
from utils.date_utils import get_formatted_date
from enums.file_enums import FileFormat
from enums.feed_enums import FeedType, FeedScope, FeedPrefix, Environment
from feed.feed_request import Feed, DEFAULT_DOWNLOAD_LOCATION
from constants.feed_constants import SUCCESS_CODE, FAILURE_CODE, PROD_CHUNK_SIZE


class TestFeed(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_token = 'Bearer v^1 ...'
        cls.test_category_1 = '1'
        cls.test_category_2 = '625'
        cls.test_marketplace = 'EBAY_US'
        cls.file_paths = []

    @classmethod
    def tearDownClass(cls):
        for file_path in cls.file_paths:
            if file_path and isfile(file_path):
                remove(file_path)

    def test_none_token(self):
        feed_req_obj = Feed(FeedType.ITEM.value, FeedScope.BOOTSTRAP.value, '220', 'EBAY_US', None)
        get_response = feed_req_obj.get()
        self.assertEqual(get_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(get_response.message)
        self.assertIsNone(get_response.file_path, 'file_path is not None in the response')

    def test_default_values(self):
        feed_req_obj = Feed(None, None, '220', 'EBAY_US', 'v^1 ...')
        self.assertEqual(feed_req_obj.feed_type, FeedType.ITEM.value)
        self.assertEqual(feed_req_obj.feed_scope, FeedScope.DAILY.value)
        self.assertTrue(feed_req_obj.token.startswith('Bearer'), 'Bearer is missing from token')
        self.assertEqual(feed_req_obj.feed_date, get_formatted_date(feed_req_obj.feed_type))
        self.assertEqual(feed_req_obj.environment, Environment.PRODUCTION.value)
        self.assertEqual(feed_req_obj.download_location, DEFAULT_DOWNLOAD_LOCATION)
        self.assertEqual(feed_req_obj.file_format, FileFormat.GZIP.value)

    def test_download_feed_invalid_path(self):
        feed_req_obj = Feed(FeedType.ITEM.value, FeedScope.BOOTSTRAP.value, '220', 'EBAY_US', 'Bearer v^1 ...',
                            download_location='../tests/test-data/test_json')
        get_response = feed_req_obj.get()
        self.assertEqual(get_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(get_response.message)
        self.assertIsNotNone(get_response.file_path, 'file_path is None in the response')

    def test_download_feed_invalid_date(self):
        feed_req_obj = Feed(FeedType.ITEM.value, FeedScope.BOOTSTRAP.value, '220', 'EBAY_US', 'Bearer v^1 ...',
                            download_location='../tests/test-data/', feed_date='2019-02-01')
        get_response = feed_req_obj.get()
        self.assertEqual(get_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(get_response.message)
        self.assertIsNotNone(get_response.file_path, 'file_path is None in the response')

    def test_download_feed_daily(self):
        test_date = get_formatted_date(FeedType.ITEM, -4)
        feed_req_obj = Feed(FeedType.ITEM.value, FeedScope.DAILY.value, self.test_category_1,
                            self.test_marketplace, self.test_token, download_location='../tests/test-data/',
                            feed_date=test_date)
        get_response = feed_req_obj.get()
        # store the file path for clean up
        self.file_paths.append(get_response.file_path)
        # assert the result
        self.assertEqual(get_response.status_code, SUCCESS_CODE)
        self.assertIsNotNone(get_response.message)
        self.assertIsNotNone(get_response.file_path, 'file_path is None')
        self.assertTrue(isfile(get_response.file_path), 'file_path is not pointing to a file. file_path: %s'
                        % get_response.file_path)
        # check the file size and name
        self.assertTrue(getsize(get_response.file_path) > 0, 'feed file is empty. file_path: %s'
                        % get_response.file_path)
        self.assertTrue(FeedPrefix.DAILY.value in get_response.file_path,
                        'feed file name does not have %s in it. file_path: %s' %
                        (FeedPrefix.DAILY.value, get_response.file_path))
        file_dir, file_name = split(abspath(get_response.file_path))
        self.assertEqual(abspath(feed_req_obj.download_location), file_dir)

    def test_download_feed_daily_bad_request(self):
        # ask for a future feed file that does not exist
        test_date = get_formatted_date(FeedType.ITEM, 5)
        feed_req_obj = Feed(FeedType.ITEM.value, FeedScope.DAILY.value, self.test_category_1,
                            self.test_marketplace, self.test_token, download_location='../tests/test-data/',
                            feed_date=test_date)
        get_response = feed_req_obj.get()
        # store the file path for clean up
        self.file_paths.append(get_response.file_path)
        # assert the result
        self.assertEqual(get_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(get_response.message)
        self.assertIsNotNone(get_response.file_path, 'file has not been created')
        self.assertTrue(isfile(get_response.file_path), 'file_path is not pointing to a file. file_path: %s'
                        % get_response.file_path)
        # check the file size and name
        self.assertTrue(getsize(get_response.file_path) == 0, 'feed file is empty. file_path: %s'
                        % get_response.file_path)
        self.assertTrue(FeedPrefix.DAILY.value in get_response.file_path,
                        'feed file name does not have %s in it. file_path: %s'
                        % (FeedPrefix.DAILY.value, get_response.file_path))
        file_dir, file_name = split(abspath(get_response.file_path))
        self.assertEqual(abspath(feed_req_obj.download_location), file_dir)

    def test_download_feed_daily_multiple_calls(self):
        feed_req_obj = Feed(FeedType.ITEM.value, FeedScope.BOOTSTRAP.value, self.test_category_2,
                            self.test_marketplace, self.test_token, download_location='../tests/test-data/')
        get_response = feed_req_obj.get()
        # store the file path for clean up
        self.file_paths.append(get_response.file_path)
        # assert the result
        self.assertEqual(get_response.status_code, SUCCESS_CODE)
        self.assertIsNotNone(get_response.message)
        self.assertIsNotNone(get_response.file_path, 'file has not been created')
        self.assertTrue(isfile(get_response.file_path), 'file_path is not pointing to a file. file_path: %s'
                        % get_response.file_path)
        # check the file size and name
        self.assertTrue(getsize(get_response.file_path) > PROD_CHUNK_SIZE, 'feed file is less than %s. file_path: %s'
                        % (PROD_CHUNK_SIZE, get_response.file_path))
        self.assertTrue(FeedPrefix.BOOTSTRAP.value in get_response.file_path,
                        'feed file name does not have %s in it. file_path: %s'
                        % (FeedPrefix.BOOTSTRAP.value, get_response.file_path))
        file_dir, file_name = split(abspath(get_response.file_path))
        self.assertEqual(abspath(feed_req_obj.download_location), file_dir)


if __name__ == '__main__':
    unittest.main()



import unittest
import sys
from utils import filter_utils
from enums.feed_enums import FeedColumn


class TestFilterUtils(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.test_column_1 = FeedColumn.PRICE_VALUE
        cls.test_column_2 = FeedColumn.ITEM_LOCATION_COUNTRIES

    def test_get_inclusive_less_query_none(self):
        query_str = filter_utils.get_inclusive_less_query(self.test_column_1, None)
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_inclusive_less_query_empty(self):
        query_str = filter_utils.get_inclusive_less_query(self.test_column_1, '')
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_inclusive_less_query(self):
        query_str = filter_utils.get_inclusive_less_query(self.test_column_1, 10)
        expected_query = '%s <= 10' % self.test_column_1
        self.assertEqual(expected_query, query_str)

    def test_get_inclusive_greater_query_none(self):
        query_str = filter_utils.get_inclusive_greater_query(self.test_column_1, None)
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_inclusive_greater_query_empty(self):
        query_str = filter_utils.get_inclusive_greater_query(self.test_column_1, '')
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_inclusive_greater_query(self):
        query_str = filter_utils.get_inclusive_greater_query(self.test_column_1, 10)
        expected_query = '%s >= 10' % self.test_column_1
        self.assertEqual(expected_query, query_str)

    def test_get_list_number_element_query_none(self):
        query_str = filter_utils.get_list_number_element_query(self.test_column_2, None)
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_list_number_element_query_empty(self):
        query_str = filter_utils.get_list_number_element_query(self.test_column_2, '')
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_list_string_element_query_none(self):
        query_str = filter_utils.get_list_string_element_query(self.test_column_2, None)
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_list_string_element_query_empty(self):
        query_str = filter_utils.get_list_string_element_query(self.test_column_2, '')
        self.assertEqual('', query_str, 'query is not an empty string')

    def test_get_list_number_element_query(self):
        query_str = filter_utils.get_list_number_element_query(self.test_column_2, [1, 2])
        expected_query = '%s IN (1,2)' % self.test_column_2
        self.assertEqual(expected_query, query_str)

    def test_get_list_string_element_query(self):
        query_str = filter_utils.get_list_string_element_query(self.test_column_2, ['CA', 'US'])
        expected_query = '%s IN (\'CA\',\'US\')' % self.test_column_2
        self.assertEqual(expected_query, query_str)

    def test_convert_to_bool_false_invalid(self):
        converted_bool = filter_utils.convert_to_bool_false('invalid')
        self.assertEqual(False, converted_bool)

    def test_convert_to_bool_false_true(self):
        converted_bool = filter_utils.convert_to_bool_false('True')
        self.assertEqual(True, converted_bool)

    def test_convert_to_bool_false_false(self):
        converted_bool = filter_utils.convert_to_bool_false('False')
        self.assertEqual(False, converted_bool)

    def convert_to_float_max_int_invalid(self):
        converted_float = filter_utils.convert_to_float_max_int('invalid')
        self.assertEqual(sys.maxsize, converted_float)

    def convert_to_float_max_int(self):
        converted_float = filter_utils.convert_to_float_max_int('1.2')
        self.assertEqual(1.2, converted_float)

    def convert_to_float_zero_invalid(self):
        converted_float = filter_utils.convert_to_float_zero('invalid')
        self.assertEqual(0, converted_float)

    def convert_to_float_zero(self):
        converted_float = filter_utils.convert_to_float_zero('1.2')
        self.assertEqual(1.2, converted_float)


if __name__ == '__main__':
    unittest.main()



import unittest
from utils import date_utils
from datetime import datetime
from enums.feed_enums import FeedType
from errors.custom_exceptions import InputDataError


class TestDateUtils(unittest.TestCase):
    def test_get_formatted_date(self):
        today_date = date_utils.get_formatted_date(FeedType.ITEM)
        try:
            datetime.strptime(today_date, '%Y%m%d')
        except ValueError:
            self.fail('Invalid date format: %s' % today_date)

    def test_validate_date_exception(self):
        with self.assertRaises(InputDataError):
            date_utils.validate_date('2019/02/01', FeedType.ITEM)

    def test_validate_date(self):
        date_utils.validate_date('20190201', FeedType.ITEM)


if __name__ == '__main__':
    unittest.main()



import unittest
import re
import utils.logging_utils as logging_utils


class TestLoggingUtils(unittest.TestCase):
    def test_log_file_name(self):
        self.assertIsNotNone(logging_utils.log_file_name)
        pattern = re.compile(logging_utils.LOG_FILE_NAME + '.\\d{4}-\\d{2}-\\d{2}' + logging_utils.LOG_FILE_EXTENSION)
        self.assertTrue(pattern.match(logging_utils.log_file_name),
                        'logging file name %s does not match the format' % logging_utils.log_file_name)


if __name__ == '__main__':
    unittest.main()



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

from aenum import Enum, unique


@unique
class FeedColumn(Enum):
    ITEM_ID = 'ItemId'  # column 0
    CATEGORY_ID = 'CategoryId'  # column 4
    SELLER_USERNAME = 'SellerUsername'  # column 6
    GTIN = 'GTIN'  # column 9
    EPID = 'EPID'  # column 12
    PRICE_VALUE = 'PriceValue'  # column 15
    ITEM_LOCATION_COUNTRIES = 'ItemLocationCountry'  # column 21
    INFERRED_EPID = 'InferredEPID'  # column 40

    def __str__(self):
        return str(self.value)


@unique
class Environment(Enum):
    PRODUCTION = 'production'
    SANDBOX = 'sandbox'

    def __str__(self):
        return str(self.value)


@unique
class FeedPrefix(Enum):
    DAILY = 'daily'
    BOOTSTRAP = 'bootstrap'

    def __str__(self):
        return str(self.value)


@unique
class FeedScope(Enum):
    DAILY = 'NEWLY_LISTED'
    BOOTSTRAP = 'ALL_ACTIVE'

    def __str__(self):
        return str(self.value)


@unique
class FeedType(Enum):
    ITEM = 'item'
    SNAPSHOT = 'item_snapshot'

    def __str__(self):
        return str(self.value)



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import logging
from datetime import datetime

LOG_FILE_NAME = 'feed-sdk-log'
LOG_FILE_EXTENSION = '.log'
LOGGING_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

log_file_name = LOG_FILE_NAME + '.' + datetime.now().strftime('%Y-%m-%d') + LOG_FILE_EXTENSION


def setup_logging():
    logging.basicConfig(filename=log_file_name, filemode='a', level=logging.DEBUG, format=LOGGING_FORMAT)



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

from datetime import datetime, timedelta
from enums.feed_enums import FeedType
from errors.custom_exceptions import InputDataError


def get_formatted_date(feed_type, day_delta=None):
    """
    :param day_delta: the day difference
    :param feed_type: item or item_snapshot
    :return: today date string in the correct format according to feed_type
    """
    delta = day_delta if day_delta else 0
    date_obj = datetime.now() + timedelta(days=delta)
    if feed_type == str(FeedType.SNAPSHOT):
        # TODO: Fix the date format
        return date_obj.strftime('%Y-%m-%dT%H:%M:%SZ')
    else:
        return date_obj.strftime('%Y%m%d')


def validate_date(feed_date, feed_type):
    """
    Validates the feed_date string format according to feed_type.
    :param feed_date: the date string feed is requested for
    :param feed_type: item or item_snapshot
    :raise InputDataError: if the date string format is not correct an InputDataError exception is raised
    """
    if feed_type == str(FeedType.SNAPSHOT):
        try:
            datetime.strptime(feed_date, '%Y-%m-%dT%H:%M:%SZ')
        except ValueError:
            raise InputDataError('Bad feed date format. Date should be in UTC format (yyyy-MM-ddThh:00:00.000Z)',
                                 feed_date)
    else:
        try:
            datetime.strptime(feed_date, '%Y%m%d')
        except ValueError:
            raise InputDataError('Bad feed date format. Date should be in yyyyMMdd format', feed_date)



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

from config.config_request import ConfigFileRequest


def filter_feed(config_path):
    cr = ConfigFileRequest(config_path)
    cr.parse_requests()
    cr.process_requests()


def download_filter_feed(config_path, token):
    cr = ConfigFileRequest(config_path)
    cr.parse_requests(token)
    cr.process_requests()


if __name__ == '__main__':
    filter_feed('../sample-config/config-file-filter')
    download_filter_feed('../sample-config/config-file-download-filter', 'v^1.1#i...')



import unittest
from enums.file_enums import FileFormat
from enums.feed_enums import FeedScope
from config.config_request import ConfigFileRequest
from feed.feed_request import DEFAULT_DOWNLOAD_LOCATION


class TestConfigRequest(unittest.TestCase):
    def test_parse_requests(self):
        cr = ConfigFileRequest('../tests/test-data/test_config')
        cr.parse_requests('Bearer v^1...')
        self.assertIsNotNone(cr.requests)
        self.assertEqual(len(cr.requests), 3)

        # first request has both feed and filter requests
        feed_req = cr.requests[0].feed_obj
        filter_req = cr.requests[0].filter_request_obj
        self.assertIsNotNone(feed_req)
        self.assertIsNotNone(filter_req)
        self.assertEqual(feed_req.category_id, '260')
        self.assertEqual(filter_req.price_lower_limit, 10)

        # second request has a feed request only
        self.assertIsNone(cr.requests[1].filter_request_obj)
        feed_req = cr.requests[1].feed_obj
        self.assertIsNotNone(feed_req)
        self.assertIsNotNone(feed_req.token)
        self.assertEqual(feed_req.category_id, '220')
        self.assertEqual(feed_req.marketplace_id, 'EBAY_US')
        self.assertEqual(feed_req.feed_date, '20190127')
        self.assertEqual(feed_req.feed_scope, FeedScope.DAILY.value)
        self.assertEqual(feed_req.download_location, DEFAULT_DOWNLOAD_LOCATION)

        # third request has a filter request only
        self.assertIsNone(cr.requests[2].feed_obj)
        filter_req = cr.requests[2].filter_request_obj
        self.assertIsNotNone(filter_req)
        self.assertEqual(filter_req.input_file_path, '/Users/[USER]/Desktop/sdk/test_bootstrap.gz')
        self.assertEqual(filter_req.leaf_category_ids, ['112529', '64619', '111694'])
        self.assertEqual(filter_req.item_location_countries, ['DE', 'GB', 'ES'])
        self.assertEqual(filter_req.any_query,
                          '(AvailabilityThresholdType=\'MORE_THAN\' AND AvailabilityThreshold=10)')
        self.assertEqual(filter_req.compression_type, FileFormat.GZIP.value)


if __name__ == '__main__':
    unittest.main()



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

class Error(Exception):
    """Base class for errors in this module."""
    pass


class AuthorizationError(Error):
    def __init__(self, msg):
        self.msg = msg


class ConfigError(Error):
    def __init__(self, msg, mark=None):
        self.msg = msg
        self.mark = mark


class FileCreationError(Error):
    def __init__(self, msg, path):
        self.msg = msg
        self.path = path


class FilterError(Error):
    def __init__(self, msg, filter_query=None):
        self.msg = msg
        self.input_data = filter_query


class InputDataError(Error):
    def __init__(self, msg, input_data=None):
        self.msg = msg
        self.input_data = input_data



import unittest
import os
import shutil
from utils import file_utils
from enums.file_enums import FileFormat
from errors.custom_exceptions import FileCreationError, InputDataError
from constants.feed_constants import SANDBOX_CHUNK_SIZE


class TestFileUtils(unittest.TestCase):
    def test_append_response_to_file(self):
        test_binary_data = b'\x01\x02\x03\x04'
        test_file_path = '../tests/test-data/testFile1'
        try:
            with open(test_file_path, 'wb') as file_obj:
                # create and append to the file
                file_utils.append_response_to_file(file_obj, test_binary_data)
        except (IOError, FileCreationError) as exp:
            # clean up
            if os.path.isfile(test_file_path):
                os.remove(test_file_path)
            self.fail(repr(exp))
        # verify that the file is created
        self.assertTrue(os.path.isfile(test_file_path), 'test file has not been created')
        # verify that file size is not zero
        self.assertTrue(os.path.getsize(test_file_path) > 0, 'the test file is empty')
        # clean up
        os.remove(test_file_path)

    def test_append_response_to_file_exception(self):
        with self.assertRaises(FileCreationError):
            file_utils.append_response_to_file(None, b'\x01')

    def test_create_and_replace_binary_file_none(self):
        with self.assertRaises(FileCreationError):
            file_utils.create_and_replace_binary_file(None)

    def test_create_and_replace_binary_file_dir(self):
        with self.assertRaises(FileCreationError):
            test_dir = os.path.expanduser('~/Desktop')
            file_utils.create_and_replace_binary_file(test_dir)

    def test_create_and_replace_binary_file_exists(self):
        test_binary_data = b'\x01\x02\x03\x04'
        test_file_path = '../tests/test-data/testFile2'
        with open(test_file_path, 'wb') as file_obj:
            file_obj.write(test_binary_data)
        # verify that the file is created
        self.assertTrue(os.path.isfile(test_file_path), 'test file has not been created')
        # verify that file size is not zero
        self.assertTrue(os.path.getsize(test_file_path) > 0, 'the test file is empty')
        # create and replace
        file_utils.create_and_replace_binary_file(test_file_path)
        # verify that the file is created
        self.assertTrue(os.path.isfile(test_file_path), 'test file has not been created')
        # verify that file size is zero
        self.assertEqual(os.path.getsize(test_file_path),  0)
        # clean up
        os.remove(test_file_path)

    def test_create_and_replace_binary_file_not_exists(self):
        test_dir_to_be_created = '../tests/test-data/testDir'
        test_file_path = os.path.join(test_dir_to_be_created, 'testFile3')
        self.assertFalse(os.path.isfile(test_file_path), 'test file exists')
        # create and replace
        file_utils.create_and_replace_binary_file(test_file_path)
        # verify that the file is created
        self.assertTrue(os.path.isfile(test_file_path), 'test file has not been created')
        # verify that file size is zero
        self.assertEqual(os.path.getsize(test_file_path),  0)
        # clean up
        shutil.rmtree(test_dir_to_be_created)

    def test_find_next_range_none_range_header(self):
        next_range = file_utils.find_next_range(None, 100)
        self.assertEqual(next_range, 'bytes=0-100')

    def test_find_next_range_none_chunk(self):
        next_range = file_utils.find_next_range('0-1000/718182376', None)
        self.assertEqual(next_range, 'bytes=1001-%s' % (SANDBOX_CHUNK_SIZE + 1001))

    def test_find_next_range(self):
        next_range = file_utils.find_next_range('1001-2001/718182376', 1000)
        self.assertEqual(next_range, 'bytes=2002-3002')

    def test_get_file_extension_none(self):
        ext = file_utils.get_extension(None)
        self.assertEqual(ext, '')

    def test_get_file_extension(self):
        ext = file_utils.get_extension(FileFormat.GZIP.value)
        self.assertEqual(ext, '.gz')

    def test_get_file_name_dir(self):
        test_dir = os.path.expanduser('../feed-sdk/tests')
        returned_dir_name = file_utils.get_file_name(test_dir)
        self.assertEqual(returned_dir_name, 'tests')

    def test_get_file_name(self):
        test_dir = os.path.expanduser('../feed-sdk/tests/test_json')
        returned_file_name = file_utils.get_file_name(test_dir)
        self.assertEqual(returned_file_name, 'test_json')

    def test_get_file_name_none(self):
        with self.assertRaises(InputDataError):
            file_utils.get_file_name(None)

    def test_get_file_name_name(self):
        test_file_name = 'abc.txt'
        self.assertEqual(file_utils.get_file_name(test_file_name), test_file_name)

    def test_read_json(self):
        json_obj = file_utils.read_json('../tests/test-data/test_json')
        self.assertIsNotNone(json_obj)
        self.assertIsNotNone(json_obj.get('requests'))


if __name__ == '__main__':
    unittest.main()



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

from aenum import Enum, unique


@unique
class ConfigField(Enum):
    FILTER_REQUEST = 'filterRequest'
    FEED_REQUEST = 'feedRequest'
    REQUESTS = 'requests'

    def __str__(self):
        return str(self.value)


@unique
class FeedField(Enum):
    MARKETPLACE_ID = 'marketplaceId'
    CATEGORY_ID = 'categoryId'
    DATE = 'date'
    SCOPE = 'feedScope'
    TYPE = 'type'
    ENVIRONMENT = 'environment'
    DOWNLOAD_LOCATION = 'downloadLocation'
    FILE_FORMAT = 'fileFormat'

    def __str__(self):
        return str(self.value)


@unique
class FilterField(Enum):
    INPUT_FILE_PATH = 'inputFilePath'
    ITEM_IDS = 'itemIds'
    LEAF_CATEGORY_IDS = 'leafCategoryIds'
    SELLER_NAMES = 'sellerNames'
    GTINS = 'gtins'
    EPIDS = 'epids'
    PRICE_LOWER_LIMIT = 'priceLowerLimit'
    PRICE_UPPER_LIMIT = 'priceUpperLimit'
    ITEM_LOCATION_COUNTRIES = 'itemLocationCountries'
    INFERRED_EPIDS = 'inferredEpids'
    ANY_QUERY = 'anyQuery'
    FILE_FORMAT = 'fileFormat'

    def __str__(self):
        return str(self.value)



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import time
import logging
import argparse
from enums.feed_enums import FeedType
from feed.feed_request import Feed
from filter.feed_filter import FeedFilterRequest
from constants.feed_constants import SUCCESS_CODE
from utils.logging_utils import setup_logging

setup_logging()
logger = logging.getLogger(__name__)

parser = argparse.ArgumentParser(prog='FeedSDK', description='Feed SDK CLI')

# date
parser.add_argument('-dt', help='the date when feed file was generated')
# l1 category
parser.add_argument('-c1', help='the l1 category id of the feed file', required=True)
# scope
parser.add_argument('-scope', help='the feed scope. Available scopes are ALL_ACTIVE or NEWLY_LISTED',
                    choices=['ALL_ACTIVE', 'NEWLY_LISTED'], default='NEWLY_LISTED')
# marketplace
parser.add_argument('-mkt', help='the marketplace id for which feed is being requested. For example - EBAY_US',
                    default='EBAY_US')
# token
parser.add_argument('-token', help='the oauth token for the consumer. Omit the word \'Bearer\'')
# environment
parser.add_argument('-env', help='environment type. Supported Environments are SANDBOX and PRODUCTION',
                    choices=['SANDBOX', 'PRODUCTION'])

# options for filtering the files
parser.add_argument('-lf', nargs='+', help='list of leaf categories which are used to filter the feed')
parser.add_argument('-sellerf', nargs='+', help='list of seller names which are used to filter the feed')
parser.add_argument('-locf', nargs='+', help='list of item locations which are used to filter the feed')
parser.add_argument('-pricelf', type=float, help='lower limit of the price range for items in the feed')
parser.add_argument('-priceuf', type=float, help='upper limit of the price range for items in the feed')
parser.add_argument('-epidf', nargs='+', help='list of epids which are used to filter the feed')
parser.add_argument('-iepidf', nargs='+', help='list of inferred epids which are used to filter the feed')
parser.add_argument('-gtinf', nargs='+', help='list of gtins which are used to filter the feed')
parser.add_argument('-itemf', nargs='+', help='list of item IDs which are used to filter the feed')
# file location
parser.add_argument('-dl', '--downloadlocation', help='override for changing the directory where files are downloaded')
parser.add_argument('--filteronly', help='filter the feed file that already exists in the default path or the path '
                                         'specified by -dl, --downloadlocation option. If --filteronly option is not '
                                         'specified, the feed file will be downloaded again', action="store_true")
# file format
parser.add_argument('-format', help='feed and filter file format. Default is gzip', default='gzip')

# any query to filter the feed file
parser.add_argument('-qf', help='any other query to filter the feed file. See Python dataframe query format')

# parse the arguments
args = parser.parse_args()


start = time.time()
if args.filteronly:
    # create the filtered file
    feed_filter_obj = FeedFilterRequest(args.downloadlocation, args.itemf, args.lf, args.sellerf, args.gtinf,
                                        args.epidf, args.pricelf, args.priceuf, args.locf, args.iepidf, args.qf,
                                        args.format)
    filter_response = feed_filter_obj.filter()
    if filter_response.status_code != SUCCESS_CODE:
        print(filter_response.message)

else:
    # download the feed file if --filteronly option is not set
    feed_obj = Feed(FeedType.ITEM.value, args.scope, args.c1, args.mkt, args.token, args.dt, args.env,
                    args.downloadlocation, args.format)
    get_response = feed_obj.get()
    if get_response.status_code != SUCCESS_CODE:
        logger.error('Exception in downloading feed. Cannot proceed\nFile path: %s\n Error message: %s\n',
                     get_response.file_path, get_response.message)
    else:
        # create the filtered file
        feed_filter_obj = FeedFilterRequest(get_response.file_path, args.itemf, args.lf, args.sellerf, args.gtinf,
                                            args.epidf, args.pricelf, args.priceuf, args.locf, args.iepidf, args.qf,
                                            args.format)
        filter_response = feed_filter_obj.filter()
        if filter_response.status_code != SUCCESS_CODE:
            print(filter_response.message)
end = time.time()
logger.info('Execution time (s): %s', str(round(end - start, 3)))
print('Execution time (s): %s' % str(round(end - start, 3)))






# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import json
from os import makedirs
from os.path import isdir, basename, splitext, exists, dirname
from errors import custom_exceptions
import constants.feed_constants as const


def append_response_to_file(file_handler, data):
    """
    Appends the given data to the existing file
    :param file_handler: the existing and open file object
    :param data: the data to be appended to the file
    :raise if there are any IO errors a FileCreationError exception is raised
    """
    try:
        file_handler.write(data)
    except (IOError, AttributeError) as exp:
        if file_handler:
            file_handler.close()
        raise custom_exceptions.FileCreationError('Error while writing in the file: %s' % repr(exp), data)


def create_and_replace_binary_file(file_path):
    """
    Creates a binary file in the given path including the file name and extension
    If the file exists, it will be replaced
    :param file_path: The path to the file including the file name and extension
    :raise: if the file is not created successfully an FileCreationError exception is raised
    """
    try:
        if not exists(dirname(file_path)):
            makedirs(dirname(file_path))
        with open(file_path, 'wb'):
            pass
    except (IOError, OSError, AttributeError) as exp:
        raise custom_exceptions.FileCreationError('IO error in creating file %s: %s' % (file_path, repr(exp)),
                                                  file_path)


def find_next_range(content_range_header, chunk_size=const.SANDBOX_CHUNK_SIZE):
    """
    Finds the next value of the Range header
    :param content_range_header: The content-range header value returned in the response, ex. 0-1000/7181823761
                                If None, the default Range header that is bytes=0-CHUNK_SIZE is returned
    :param chunk_size: The chunk size in bytes. If not provided, the default chunk size is used
    :return: The next value of the Range header in the format of bytes=lower-upper or empty string if no data is left
    :raise: If the input content-range value is not correct an InputDataError exception is raised
    """
    chunk = chunk_size if chunk_size else const.SANDBOX_CHUNK_SIZE
    if content_range_header is None:
        return const.RANGE_PREFIX + '0-' + str(chunk)
    else:
        try:
            # ex. content-range : 0-1000/7181823761
            range_components = content_range_header.split('/')
            total_size = int(range_components[1])
            bounds = range_components[0].split('-')
            upper_bound = int(bounds[1]) + 1
            if upper_bound > total_size:
                return ''
            return const.RANGE_PREFIX + str(upper_bound) + '-' + str(upper_bound + chunk)
        except Exception:
            raise custom_exceptions.InputDataError('Bad content-range header format: %s' % content_range_header,
                                                   content_range_header)


def get_extension(file_type):
    """
    Returns file extension including '.' according to the given file type
    :param file_type: format of the file such as gzip
    :return: extension of the file such as '.gz'
    """
    if not file_type:
        return ''
    if file_type.lower() == 'gz' or file_type.lower() == 'gzip':
        return '.gz'


def get_file_name(name_or_path):
    """
    Finds name of the file from the given file path or name
    :param name_or_path: name or path to the file
    :return: file name
    """
    if not name_or_path:
        raise custom_exceptions.InputDataError('Bad file name or directory %s' % name_or_path, name_or_path)
    if isdir(name_or_path):
        base = basename(name_or_path)
        return splitext(base)
    elif '/' in name_or_path:
        return name_or_path[name_or_path.rfind('/') + 1:]
    else:
        return name_or_path


def read_json(file_path):
    """
    Reads json from a file and returns a json object
    :param file_path: the path to the file
    :return: a json object
    """
    with open(file_path) as config_file:
        json_obj = json.load(config_file)
    return json_obj



import unittest
from os import remove
from os.path import isfile
from filter.feed_filter import FeedFilterRequest
from enums.file_enums import FileFormat, FileEncoding
from constants.feed_constants import DATA_FRAME_CHUNK_SIZE, SUCCESS_CODE, FAILURE_CODE


class TestFeed(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.test_file_path = '../tests/test-data/test_bootstrap_feed_260_3'
        cls.test_any_query = 'AvailabilityThresholdType=\'MORE_THAN\' AND AvailabilityThreshold=10'

    def test_default_values(self):
        filter_request = FeedFilterRequest(self.test_file_path)
        self.assertIsNone(filter_request.item_ids)
        self.assertIsNone(filter_request.leaf_category_ids)
        self.assertIsNone(filter_request.seller_names)
        self.assertIsNone(filter_request.gtins)
        self.assertIsNone(filter_request.epids)
        self.assertIsNone(filter_request.price_upper_limit)
        self.assertIsNone(filter_request.price_lower_limit)
        self.assertIsNone(filter_request.item_location_countries)
        self.assertIsNone(filter_request.inferred_epids)
        self.assertIsNone(filter_request.item_location_countries)
        self.assertIsNone(filter_request.any_query)
        self.assertIsNone(filter_request.filtered_file_path)
        self.assertEqual(filter_request.compression_type, FileFormat.GZIP.value)
        self.assertEqual(filter_request.separator, '\t')
        self.assertEqual(filter_request.encoding, FileEncoding.UTF8.value)
        self.assertEqual(filter_request.rows_chunk_size, DATA_FRAME_CHUNK_SIZE)
        self.assertEqual(filter_request.number_of_records, 0)
        self.assertEqual(filter_request.number_of_filtered_records, 0)
        self.assertEqual(len(filter_request.queries), 0)

    def test_any_query_format(self):
        filter_request = FeedFilterRequest(self.test_file_path, any_query=self.test_any_query)
        self.assertEqual(filter_request.any_query, '(' + self.test_any_query + ')')

    def test_none_file_path(self):
        filter_request = FeedFilterRequest(None)
        filter_response = filter_request.filter()
        self.assertEqual(filter_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(filter_response.message)
        self.assertIsNone(filter_response.file_path)
        self.assertEqual(len(filter_response.applied_filters), 0)

    def test_dir_file_path(self):
        filter_request = FeedFilterRequest('../tests/test-data')
        filter_response = filter_request.filter()
        self.assertEqual(filter_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(filter_response.message)
        self.assertIsNone(filter_response.file_path)
        self.assertEqual(len(filter_response.applied_filters), 0)

    def test_no_query(self):
        filter_request = FeedFilterRequest(self.test_file_path)
        filter_response = filter_request.filter()
        self.assertEqual(filter_response.status_code, FAILURE_CODE)
        self.assertIsNotNone(filter_response.message)
        self.assertIsNone(filter_response.file_path)
        self.assertEqual(len(filter_response.applied_filters), 0)

    def test_apply_filters(self):
        filter_request = FeedFilterRequest(self.test_file_path, price_upper_limit=10, any_query=self.test_any_query)
        filter_response = filter_request.filter(keep_db=False)
        self.assertEqual(filter_response.status_code, SUCCESS_CODE)
        self.assertIsNotNone(filter_response.message)

        self.assertEqual(len(filter_request.queries), 2)
        self.assertEqual(len(filter_response.applied_filters), 2)

        self.assertTrue(filter_request.number_of_records > 0)
        self.assertTrue(filter_request.number_of_filtered_records > 0)

        self.assertIsNotNone(filter_request.filtered_file_path)
        self.assertTrue(isfile(filter_request.filtered_file_path))
        self.assertEqual(filter_request.filtered_file_path, filter_response.file_path)
        # clean up
        remove(filter_request.filtered_file_path)


if __name__ == '__main__':
    unittest.main()



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import certifi
import urllib3
import json
import logging
from os import path
from utils import file_utils, date_utils
import constants.feed_constants as const
from filter.feed_filter import GetFeedResponse
from enums.file_enums import FileFormat
from enums.feed_enums import FeedType, FeedScope, FeedPrefix, Environment
from errors.custom_exceptions import InputDataError, FileCreationError
from utils.logging_utils import setup_logging

setup_logging()
logger = logging.getLogger(__name__)

DEFAULT_DOWNLOAD_LOCATION = path.expanduser('~/Desktop/feed-sdk')


class Feed(object):
    def __init__(self, feed_type, feed_scope, category_id, marketplace_id, token, feed_date=None,
                 environment=Environment.PRODUCTION.value, download_location=None, file_format=FileFormat.GZIP.value):
        self.token = const.TOKEN_BEARER_PREFIX + token if (token and not token.startswith('Bearer')) else token
        self.feed_type = feed_type.lower() if feed_type else FeedType.ITEM.value
        self.feed_scope = feed_scope.upper() if feed_scope else FeedScope.DAILY.value
        self.category_id = category_id
        self.marketplace_id = marketplace_id
        self.environment = environment if environment else Environment.PRODUCTION.value
        self.download_location = download_location if download_location else DEFAULT_DOWNLOAD_LOCATION
        self.file_format = file_format if file_format else FileFormat.GZIP.value
        self.feed_date = feed_date if feed_date else date_utils.get_formatted_date(feed_type)

    def __str__(self):
        return '[feed_type= %s, feed_scope= %s, category_id= %s, marketplace_id= %s, feed_date= %s, ' \
               'environment= %s,  download_location= %s, file_format= %s, token= %s]' % (self.feed_type,
                                                                                         self.feed_scope,
                                                                                         self.category_id,
                                                                                         self.marketplace_id,
                                                                                         self.feed_date,
                                                                                         self.environment,
                                                                                         self.download_location,
                                                                                         self.file_format,
                                                                                         self.token)

    def get(self):
        """
        :return: GetFeedResponse
        """
        logger.info(
            'Downloading... \ncategoryId: %s | marketplace: %s | date: %s | feed_scope: %s | environment: %s \n',
            self.category_id, self.marketplace_id, self.feed_date, self.feed_scope, self.environment)
        if not self.token:
            return GetFeedResponse(const.FAILURE_CODE, 'No token has been provided', None, None, None)
        if path.exists(self.download_location) and not path.isdir(self.download_location):
            return GetFeedResponse(const.FAILURE_CODE, 'Download location is not a directory', self.download_location,
                                   None, None)
        try:
            date_utils.validate_date(self.feed_date, self.feed_type)
        except InputDataError as exp:
            return GetFeedResponse(const.FAILURE_CODE, exp.msg, self.download_location, None, None)
        # generate the absolute file path
        file_name = self.__generate_file_name()
        file_path = path.join(self.download_location, file_name)
        # Create an empty file in the given path
        try:
            file_utils.create_and_replace_binary_file(file_path)
            with open(file_path, 'wb') as file_obj:
                # Get the feed file data
                result_code, message = self.__invoke_request(file_obj)
                return GetFeedResponse(result_code, message, file_path, None, None)
        except IOError as exp:
            return GetFeedResponse(const.FAILURE_CODE, 'Could not open file %s : %s' % (file_path, repr(exp)),
                                   file_path, None, None)
        except (InputDataError, FileCreationError) as exp:
            return GetFeedResponse(const.FAILURE_CODE, exp.msg, file_path, None, None)

    def __invoke_request(self, file_handler):
        # initialize API call counter
        api_call_counter = 0
        # Find max chunk size
        chunk_size = self.__find_max_chunk_size()
        logger.info('Chunk size: %s\n', chunk_size)
        # The initial request Range header is bytes=0-CHUNK_SIZE
        headers = {const.MARKETPLACE_HEADER: self.marketplace_id,
                   const.AUTHORIZATION_HEADER: self.token,
                   const.CONTENT_TYPE_HEADER: const.APPLICATION_JSON,
                   const.ACCEPT_HEADER: const.APPLICATION_JSON,
                   const.RANGE_HEADER: const.RANGE_PREFIX + '0-' + str(chunk_size)}
        parameters, endpoint = self.__get_query_parameters_and_base_url()
        http_manager = urllib3.PoolManager(timeout=const.REQUEST_TIMEOUT,
                                           retries=urllib3.Retry(const.REQUEST_RETRIES,
                                                                 backoff_factor=const.BACK_OFF_TIME),
                                           cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
        # Initial request
        feed_response = http_manager.request('GET', endpoint, parameters, headers)
        # increase and print API call counter
        api_call_counter = api_call_counter + 1
        logger.info('API call #%s\n', api_call_counter)
        # Get the status code
        status_code = feed_response.status
        # Append the data to the file, might raise an exception
        if status_code == 200:
            file_utils.append_response_to_file(file_handler, feed_response.data)
            return const.SUCCESS_CODE, const.SUCCESS_STR
        while status_code == 206:
            # Append the data to the file, might raise an exception
            file_utils.append_response_to_file(file_handler, feed_response.data)
            headers[const.RANGE_HEADER] = file_utils.find_next_range(feed_response.headers[const.CONTENT_RANGE_HEADER],
                                                                     chunk_size)
            # check if we have reached the end of the file
            if not headers[const.RANGE_HEADER]:
                break
            # Send another request
            feed_response = http_manager.request('GET', endpoint, parameters, headers)
            # increase and print API call counter
            api_call_counter = api_call_counter+1
            logger.info('API call #%s\n', api_call_counter)
            # Get the status code
            status_code = feed_response.status
        if status_code == 206 and not headers[const.RANGE_HEADER]:
            return const.SUCCESS_CODE, const.SUCCESS_STR
        json_response = json.loads(feed_response.data.decode('utf-8'))
        return const.FAILURE_CODE, json_response.get('errors')

    def __get_query_parameters_and_base_url(self):
        # Base URL
        base_url = self.__find_base_url()
        base_url = base_url + str(FeedType.ITEM)
        # Common query parameter
        fields = {const.QUERY_CATEGORY_ID: self.category_id}
        # Snapshot feed
        if self.feed_type == str(FeedType.SNAPSHOT):
            fields.update({const.QUERY_SNAPSHOT_DATE: self.feed_date})
            base_url = const.FEED_API_PROD_URL + str(FeedType.SNAPSHOT)
            return fields, base_url
        # Daily or bootstrap feed
        if self.feed_scope == str(FeedScope.DAILY):
            fields.update({const.QUERY_SCOPE: self.feed_scope,
                           const.QUERY_DATE: self.feed_date})
        elif self.feed_scope == str(FeedScope.BOOTSTRAP):
            fields.update({const.QUERY_SCOPE: self.feed_scope})
        return fields, base_url

    def __find_base_url(self):
        if self.environment.lower() == str(Environment.PRODUCTION):
            return const.FEED_API_PROD_URL
        return const.FEED_API_SANDBOX_URL

    def __find_max_chunk_size(self):
        if self.environment.lower() == str(Environment.PRODUCTION):
            return const.PROD_CHUNK_SIZE
        return const.SANDBOX_CHUNK_SIZE

    def __generate_file_name(self):
        if str(FeedScope.BOOTSTRAP) == self.feed_scope:
            feed_prefix = str(FeedPrefix.BOOTSTRAP)
        elif str(FeedScope.DAILY) == self.feed_scope:
            feed_prefix = str(FeedPrefix.DAILY)
        else:
            raise InputDataError('Unknown feed scope', self.feed_scope)
        file_name = str(FeedType.ITEM) + '_' + feed_prefix + '_' + str(self.category_id) + '_' + self.feed_date + \
            '_' + self.marketplace_id + file_utils.get_extension(self.file_format)
        return file_name



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import time
import logging
import pandas as pd
from os import remove
from sqlalchemy import create_engine
from collections import namedtuple
from os.path import split, abspath, join, isfile
from utils import filter_utils
from utils.file_utils import get_extension

from enums.feed_enums import FeedColumn
from enums.file_enums import FileEncoding, FileFormat
import constants.feed_constants as const
from utils.logging_utils import setup_logging

setup_logging()
logger = logging.getLogger(__name__)

Response = namedtuple('Response', 'status_code message file_path applied_filters')
GetFeedResponse = namedtuple('GetFeedResponse', Response._fields + ('errors',))

BOOL_COLUMNS = {'ImageAlteringProhibited', 'ReturnsAccepted'}
# using float64 for integer columns as well as the workaround for NAN values
FLOAT_COLUMNS = {'AvailabilityThreshold', 'EstimatedAvailableQuantity',
                 'PriceValue', 'ReturnPeriodValue'}
IGNORE_COLUMNS = {'AdditionalImageUrls', 'ImageUrl', 'Title'}

DB_FILE_NAME = 'sqlite_feed_sdk.db'
DB_TABLE_NAME = 'feed'


class FeedFilterRequest(object):
    def __init__(self, input_file_path, item_ids=None, leaf_category_ids=None, seller_names=None, gtins=None,
                 epids=None, price_lower_limit=None, price_upper_limit=None, item_location_countries=None,
                 inferred_epids=None, any_query=None, compression_type=FileFormat.GZIP.value, separator='\t',
                 encoding=FileEncoding.UTF8.value, rows_chunk_size=const.DATA_FRAME_CHUNK_SIZE):
        self.input_file_path = input_file_path
        self.item_ids = item_ids
        self.leaf_category_ids = leaf_category_ids
        self.seller_names = seller_names
        self.gtins = gtins
        self.epids = epids
        self.price_lower_limit = price_lower_limit
        self.price_upper_limit = price_upper_limit
        self.item_location_countries = item_location_countries
        self.inferred_epids = inferred_epids
        self.any_query = '(%s)' % any_query if any_query else None
        self.compression_type = compression_type if compression_type else FileFormat.GZIP.value
        self.separator = separator if separator else '\t'
        self.encoding = encoding if encoding else FileEncoding.UTF8.value
        self.rows_chunk_size = rows_chunk_size if rows_chunk_size else const.DATA_FRAME_CHUNK_SIZE
        self.__filtered_file_path = None
        self.__number_of_records = 0
        self.__number_of_filtered_records = 0
        self.__queries = []

    def __str__(self):
        return '[input_file_path= %s, item_ids= %s, leaf_category_ids= %s, seller_names= %s, gtins= %s, ' \
               'epids= %s,  price_lower_limit= %s, price_upper_limit= %s, item_location_countries= %s, ' \
               'inferred_epids= %s,  any_query= %s, compression_type= %s, separator= %s, encoding= %s]' % \
               (self.input_file_path,
                self.item_ids,
                self.leaf_category_ids,
                self.seller_names,
                self.gtins,
                self.epids,
                self.price_lower_limit,
                self.price_upper_limit,
                self.item_location_countries,
                self.inferred_epids,
                self.any_query,
                self.compression_type,
                self.separator,
                self.encoding)

    @property
    def filtered_file_path(self):
        return self.__filtered_file_path

    @property
    def number_of_records(self):
        return self.__number_of_records

    @property
    def number_of_filtered_records(self):
        return self.__number_of_filtered_records

    @property
    def queries(self):
        return self.__queries

    def __append_query(self, query_str):
        if query_str:
            self.__queries.append(query_str)

    def filter(self, column_name_list=None, keep_db=False):
        logger.info('Filtering... \nInput file: %s', self.input_file_path)

        self.__append_query(self.any_query)
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.ITEM_ID, self.item_ids))
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.CATEGORY_ID, self.leaf_category_ids))
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.SELLER_USERNAME, self.seller_names))
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.GTIN, self.gtins))
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.EPID, self.epids))
        self.__append_query(filter_utils.get_inclusive_greater_query(FeedColumn.PRICE_VALUE, self.price_lower_limit))
        self.__append_query(filter_utils.get_inclusive_less_query(FeedColumn.PRICE_VALUE, self.price_upper_limit))
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.INFERRED_EPID, self.inferred_epids))
        self.__append_query(filter_utils.get_list_string_element_query(FeedColumn.ITEM_LOCATION_COUNTRIES,
                                                                       self.item_location_countries))
        query_str = None
        if self.__queries:
            query_str = ' AND '.join(self.__queries)
        if not self.input_file_path or not isfile(self.input_file_path):
            return Response(const.FAILURE_CODE,
                            'Input file is a directory or does not exist. Cannot filter. Aborting...',
                            self.filtered_file_path, self.queries)
        if not query_str:
            return Response(const.FAILURE_CODE, 'No filters have been specified. Cannot filter. Aborting...',
                            self.filtered_file_path, self.queries)
        # create the data frame
        filtered_data = self.__read_chunks_gzip_file(query_str, column_name_list, keep_db)
        if not filtered_data.empty:
            self.__save_filtered_data_frame(filtered_data)
        else:
            logger.error('No filtered feed file created')
        return Response(const.SUCCESS_CODE, const.SUCCESS_STR, self.filtered_file_path, self.queries)

    def __derive_filtered_file_path(self):
        file_path, full_file_name = split(abspath(self.input_file_path))
        file_name = full_file_name.split('.')[0]
        time_milliseconds = int(time.time() * 1000)
        filtered_file_path = join(file_path, file_name + '-filtered-' + str(time_milliseconds) +
                                  get_extension(self.compression_type))
        return filtered_file_path

    def __read_chunks_gzip_file(self, query_str, column_name_list, keep_db):
        disk_engine = create_engine('sqlite:///'+DB_FILE_NAME)
        chunk_num = 0
        columns_to_process, data_types = self.__get_cols_and_type_dict()
        cols = column_name_list if column_name_list else columns_to_process
        start = time.time()
        for chunk_df in pd.read_csv(self.input_file_path, header=0,
                                    compression=self.compression_type, encoding=self.encoding, usecols=cols,
                                    sep=self.separator, quotechar='"', lineterminator='\n', skip_blank_lines=True,
                                    skipinitialspace=True, error_bad_lines=False, index_col=False,
                                    chunksize=self.rows_chunk_size, dtype=data_types,
                                    converters={'AvailabilityThreshold': filter_utils.convert_to_float_max_int,
                                                'EstimatedAvailableQuantity': filter_utils.convert_to_float_max_int,
                                                'PriceValue': filter_utils.convert_to_float_zero,
                                                'ReturnPeriodValue': filter_utils.convert_to_float_zero,
                                                'ImageAlteringProhibited': filter_utils.convert_to_bool_false,
                                                'ReturnsAccepted': filter_utils.convert_to_bool_false}):
            self.__number_of_records = self.__number_of_records + len(chunk_df.index)
            chunk_num = chunk_num + 1
            chunk_df.to_sql(DB_TABLE_NAME, disk_engine, if_exists='append', index=False)
        execution_time = time.time() - start
        logger.info('Loaded %s records in %s (s) %s (m)', self.__number_of_records, str(round(execution_time, 3)),
                     str(round(execution_time / 60, 3)))
        # apply query
        sql_string = '''SELECT * From %s WHERE %s ''' % (DB_TABLE_NAME, query_str)

        start = time.time()
        query_result_df = pd.read_sql_query(sql_string, disk_engine)
        execution_time = time.time() - start
        self.__number_of_filtered_records = len(query_result_df.index)
        logger.info('Filtered %s records in %s (s) %s (m)', self.number_of_filtered_records,
                     str(round(execution_time, 3)),
                     str(round(execution_time / 60, 3)))
        # remove the created db file
        if not keep_db:
            remove(DB_FILE_NAME)
        return query_result_df

    def __save_filtered_data_frame(self, data_frame):
        self.__filtered_file_path = self.__derive_filtered_file_path()
        start = time.time()
        data_frame.to_csv(self.__filtered_file_path, sep=self.separator, na_rep='', header=True, index=False, mode='w',
                          encoding=self.encoding, compression=self.compression_type, quotechar='"',
                          line_terminator='\n', doublequote=True, escapechar='\\', decimal='.')
        execution_time = time.time() - start
        logger.info('Saved %s records in %s (s) %s (m)', self.number_of_filtered_records,
                    str(round(execution_time, 3)),
                    str(round(execution_time / 60, 3)))

    def __get_cols_and_type_dict(self):
        all_columns = pd.read_csv(self.input_file_path, nrows=1, sep=self.separator,
                                  compression=self.compression_type).columns.tolist()
        type_dict = {}
        cols = []
        for col_name in all_columns:
            # Ignoring due to possibility of comma character in the value and breaking the parser
            if col_name in IGNORE_COLUMNS:
                continue
            else:
                cols.append(col_name)
                if col_name not in BOOL_COLUMNS and col_name not in FLOAT_COLUMNS:
                    type_dict[col_name] = 'object'
        return cols, type_dict



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ebay_feedsdk',
 'ebay_feedsdk.config',
 'ebay_feedsdk.constants',
 'ebay_feedsdk.enums',
 'ebay_feedsdk.errors',
 'ebay_feedsdk.examples',
 'ebay_feedsdk.feed',
 'ebay_feedsdk.filter',
 'ebay_feedsdk.tests',
 'ebay_feedsdk.utils']

package_data = \
{'': ['*'],
 'ebay_feedsdk': ['sample-config/*'],
 'ebay_feedsdk.tests': ['test-data/*']}

setup_kwargs = {
    'name': 'ebay-feedsdk',
    'version': '0.1.1',
    'description': 'Port of https://github.com/eBay/FeedSDK-Python to python3',
    'long_description': 'Feed SDK\n==========\nPython SDK for downloading and filtering item feed files\n\nForked from [https://github.com/eBay/FeedSDK-Python](https://github.com/eBay/FeedSDK-Python) and ported to python3\n\nAvailable as PyPI package under https://pypi.org/project/ebay-feedsdk/\n\n\nTable of contents\n==========\n* [Summary](#summary)\n* [Setup](#setup)\n    - [Setting up in the local environment](#setting-up-in-the-local-environment)\n* [Downloading feed files](#downloading-feed-files)\n    - [Customizing download location](#customizing-download-location)\n* [Filtering feed files](#filtering-feed-files)\n    - [Available filters](#available-filters)\n    - [Combining filter criteria](#combining-filter-criteria)\n    - [Additional filter arguments](#additional-filter-arguments)\n* [Schemas](#schemas)\n    - [GetFeedResponse](#getfeedresponse)\n    - [Response](#response)\n* [Logging](#logging)\n* [Usage](#usage)\n    - [Using command line options](#using-command-line-options)\n    - [Using config file driven approach](#using-config-file-driven-approach)\n    - [Using function calls](#using-function-calls)\n        - [Code samples](#examples)\n* [Performance](#performance)\n* [Important notes](#important-notes)\n\n# Summary\n\nSimilar to [Java Feed SDK](https://github.com/eBay/FeedSDK), this Python SDK facilitates download and filtering of eBay\'s item feed files provided through public [Feed API](https://developer.ebay.com/api-docs/buy/feed/overview.html).\n\nThe feed SDK provides a simple interface to -\n* [Download](#downloading-feed-files)\n* [Filter](#filtering-feed-files)\n\n# Setup\n\nThe the entire repository can be cloned/forked and changes can be made. You are most welcome to collaborate and enhance the existing code base.\n\n## Setting up in the local environment\n\nFor setting up the project in your local environment\n* Clone or download the repository\n* Install the requirements\nTo set up your environment, please see the requirements listed in [requirements.txt](https://github.com/eBay/FeedSDK-Python/blob/master/requirements.txt). You can run $ pip install -r requirements.txt command to install all the requirements.\n\n\n## Downloading feed files\nThe feed files can be as big as several gigabytes. Feed API supports downloading such big feed files in chunks. Chunk size is 100 MB in production environment and is 10 MB in soundbox environment.\n\nThe SDK abstracts the complexity involved in calculating the request header \'__range__\' based on the response header \'__content-range__\' and downloads and appends all the chunks until the whole feed file is downloaded.\n\nTo download a feed file in production which is -\n* __bootstrap__ : (feed_scope = ALL_ACTIVE)\n* __L1 category 1__ : (category_id = 220)\n* __marketplace US__ : (X-EBAY-C-MARKETPLACE-ID: EBAY_US)\ninstantiate a Feed object and call get() function\n\n```\nfeed_obj = Feed(feed_type=\'item\', feed_scope=\'ALL_ACTIVE\', category_id=\'220\', \n\t\t\t       marketplace_id=\'EBAY_US\', token=<TOKEN>, environment=\'PRODUCTION\')\nresult_code, api_status_code, file_path = feed_obj.get()\n\n```\nThe __filePath__ denotes the location where the file was downloaded.\n\n### Customizing download location\n\nThe default download location is ~/Desktop/feed-sdk directory. If the directory does not exist, it will be created.\nThe download location can be changed by specifying the optional \'download_location\' argument when instantiating Feed.\nThe download location should point to a directory. If the directory does not exist, it will be created.\nFor example, to download to the location __/tmp/feed__ - \n\n```\nfeed_obj = Feed(feed_type=\'item\', feed_scope=\'ALL_ACTIVE\', category_id=\'220\', \n                               marketplace_id=\'EBAY_US\', token=<TOKEN>, environment=\'PRODUCTION\',\n\t\t\t       download_location=\'/tmp/feed\')\n```\n---\n\n## Filtering feed files\n\n### Available filters\nThe SDK provides the capability to filter the feed files based on :-\n* List of leaf category ids\n* List of seller usernames\n* List of item locations\n* List of item IDs\n* List of EPIDs\n* List of inferred EPIDs\n* List of GTINs\n* Price range\n* Any other SQL query\n\nOn successful completion of a filter operation, a new __filtered__ file is created in the same directory as the feed file\'s.\n\nTo filter a feed file on leaf category IDs create an object of FeedFilterRequest and call filter() function - \n```\nfeed_filter_obj = FeedFilterRequest(input_fila_path=<absolute path to the feed file>, \n                                    leaf_category_ids=<list of leaf category IDs>)\nfile_path = feed_filter_obj.filter()\n\n```\n\nTo filter on availability threshold type and availability threshold via any_query parameter\n```\nfeed_filter_obj = FeedFilterRequest(input_fila_path=<absolute path to the feed file>,\n                                    any_query=\'AvailabilityThresholdType=\\\'MORE_THAN\\\' AND AvailabilityThreshold==10\')\nfile_path = feed_filter_obj.filter()\n\n```\n\nThe __file_path__ denotes the location of the filtered file. The file_path value can also be read by filter_request.filtered_file_path.\n\n### Combining filter criteria\n\nThe SDK provides the freedom to combine the filter criteria.\n\nTo filter on leaf category IDs and seller user names for listings in the price range of 1 to 100\n\n```\nfeed_filter_obj = FeedFilterRequest(input_fila_path=<absolute path to the feed file>,\n                                    leaf_category_ids=<list of leaf category IDs>, \n\t\t\t\t    seller_names=<list of seller names>,\n                                    price_lower_limit=1, price_upper_limit=100)\nfile_path = feed_filter_obj.filter()\n\n```\n\nTo filter on item location countries for listings that have more than 10 items available\n\n```\nfeed_filter_obj = FeedFilterRequest(input_fila_path=<absolute path to the feed file>,\n                                    item_location_countries=<list of item location countries>, \n                                    any_query=\'AvailabilityThresholdType=\\\'MORE_THAN\\\' AND AvailabilityThreshold=10\')\nfile_path = feed_filter_obj.filter()\n\n```\n\n### Additional filter arguments\nWhen filter function is called, feed data is loaded into a sqlite DB.\nIf keep_db=True argument is passed to filter function, the sqlite db file is kept in the current directory with name sqlite_feed_sdk.db, otherwise it will be deleted after the program execution.\n\nBy default all the columns except Title, ImageUrl, and AdditionalImageUrls are processed. This behaviour can be changed by passing column_name_list argument to filter function and changing IGNORE_COLUMNS set in feed_filter.py. \n\n---\n### Schemas\nThis section provides more detail on what information is contained within the objects returned from the SDK function calls.\n\n### GetFeedResponse\n\nAn instance of GetFeedResponse named tuple is returned from the feed_obj.get() function.\n\n```\n  int status_ode\n  String message\n  String file_path\n  List errors\n\n```\n\n| Field name | Description \n|---|---|\n| status_code | int: 0 indicates a successful response. Any non zero value indicates an error\n| message | String: Detailed information on the status\n| file_path | String: Absolute path of the location of the resulting file\n| errors | List: Detailed error information\n\n\n### Response \n\nAn instance of Response named tuple is returned from feed_filter_object.filter() function.\n\n```\n  int status_code\n  String message\n  String file_path\n  List applied_filters\n```\n| Field name | Description \n|---|---|\n| status_code | int: 0 indicates a successful response. Any non zero value indicates an error\n| message | String: Detailed information on the status\n| file_path | String: Absolute path of the location of the resulting file\n| applied_filters | List: List of queries applied\n\n---\n## Logging\n\nLog files are created in the current directory.\n\n__Ensure that appropriate permissions are present to write to the directory__\n\n* The current log file name is : feed-sdk-log.log\n* Rolling log files are created per day with the pattern : feed-sdk-log.{yyyy-MM-dd}.log\n\n---\n## Usage\n\nThe following sections describe the different ways in which the SDK can be used\n\n### Using command line options\n\nAll the capabilities of the SDK can be invoked using the command line.\n\nTo see the available options and filters , use \'--help\'\n```\nusage: FeedSDK [-h] [-dt DT] -c1 C1 [-scope {ALL_ACTIVE,NEWLY_LISTED}]\n               [-mkt MKT] [-token TOKEN] [-env {SANDBOX,PRODUCTION}]\n               [-lf LF [LF ...]] [-sellerf SELLERF [SELLERF ...]]\n               [-locf LOCF [LOCF ...]] [-pricelf PRICELF] [-priceuf PRICEUF]\n               [-epidf EPIDF [EPIDF ...]] [-iepidf IEPIDF [IEPIDF ...]]\n               [-gtinf GTINF [GTINF ...]] [-itemf ITEMF [ITEMF ...]]\n               [-dl DOWNLOADLOCATION] [--filteronly] [-format FORMAT] [-qf QF]\n\nFeed SDK CLI\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -dt DT                the date when feed file was generated\n  -c1 C1                the l1 category id of the feed file\n  -scope {ALL_ACTIVE,NEWLY_LISTED}\n                        the feed scope. Available scopes are ALL_ACTIVE or\n                        NEWLY_LISTED\n  -mkt MKT              the marketplace id for which feed is being requested.\n                        For example - EBAY_US\n  -token TOKEN          the oauth token for the consumer. Omit the word\n                        \'Bearer\'\n  -env {SANDBOX,PRODUCTION}\n                        environment type. Supported Environments are SANDBOX\n                        and PRODUCTION\n  -lf LF [LF ...]       list of leaf categories which are used to filter the\n                        feed\n  -sellerf SELLERF [SELLERF ...]\n                        list of seller names which are used to filter the feed\n  -locf LOCF [LOCF ...]\n                        list of item locations which are used to filter the\n                        feed\n  -pricelf PRICELF      lower limit of the price range for items in the feed\n  -priceuf PRICEUF      upper limit of the price range for items in the feed\n  -epidf EPIDF [EPIDF ...]\n                        list of epids which are used to filter the feed\n  -iepidf IEPIDF [IEPIDF ...]\n                        list of inferred epids which are used to filter the\n                        feed\n  -gtinf GTINF [GTINF ...]\n                        list of gtins which are used to filter the feed\n  -itemf ITEMF [ITEMF ...]\n                        list of item IDs which are used to filter the feed\n  -dl DOWNLOADLOCATION, --downloadlocation DOWNLOADLOCATION\n                        override for changing the directory where files are\n                        downloaded\n  --filteronly          filter the feed file that already exists in the\n                        default path or the path specified by -dl,\n                        --downloadlocation option. If --filteronly option is\n                        not specified, the feed file will be downloaded again\n  -format FORMAT        feed and filter file format. Default is gzip\n  -qf QF                any other query to filter the feed file. See Python\n                        dataframe query format\n```\nFor example, to use the command line options to\n\nDownload and filter feed files using token\n```\npython feed_cli.py -c1 3252 -scope ALL_ACTIVE -mkt EBAY_DE -env PRODUCTION -qf "AvailabilityThreshold=10" -locf IT GB -dl DIR -token xxx\n```\n\nFilter feed files, no token is needed\n```\npython feed_cli.py --filteronly -c1 260 -pricelf 5 -priceuf 20 -dl FILE_PATH\n```\n\n### Using config file driven approach\n\nAll the capabilities of the SDK can be leveraged via a config file.\nThe feed file download and filter parameters can be specified in the config file for multiple files, and SDK will process them sequentially.\n\nThe structure of the config file\n\n```\n{\n  "requests": [\n    {\n      "feedRequest": {\n        "categoryId": "260",\n        "marketplaceId": "EBAY_US",\n        "feedScope": "ALL_ACTIVE",\n        "type": "ITEM"\n      },\n      "filterRequest": {\n        "itemLocationCountries": [\n          "US",\n          "HK",\n          "CA"\n        ],\n        "priceLowerLimit": 10.0,\n        "priceUpperLimit": 100.0\n      }\n    },\n    {\n      "feedRequest": {\n        "categoryId": "220",\n        "marketplaceId": "EBAY_US",\n        "date": "20190127",\n        "feedScope": "NEWLY_LISTED",\n        "type": "ITEM"\n      }\n    },\n    {\n      "filterRequest": {\n        "inputFilePath": "<Absolute file path to the feed file>",\n        "leafCategoryIds": [\n          "112529",\n          "64619",\n          "111694"\n        ],\n        "itemLocationCountries": [\n          "DE",\n          "GB",\n          "ES"\n        ],\n        "anyQuery": "AvailabilityThresholdType=\'MORE_THAN\' AND AvailabilityThreshold=10",\n        "fileFormat" : "gzip"\n      }\n    }\n  ]\n}\n```\nAn example of using the SDK through a config file is located at \n\n[Example config file - 1](https://github.com/eBay/FeedSDK-Python/blob/master/sample-config/config-file-download)\n\n[Example config file - 2](https://github.com/eBay/FeedSDK-Python/blob/master/sample-config/config-file-download-filter)\n\n[Example config file - 3](https://github.com/eBay/FeedSDK-Python/blob/master/sample-config/config-file-filter)\n\n[Example config file - 4](https://github.com/eBay/FeedSDK-Python/blob/master/sample-config/config-file-query-only)\n\n### Using function calls\n\nSamples showing the usage of available operations and filters.\n\n#### Examples\nAll the examples are located [__here__](https://github.com/eBay/FeedSDK-Python/tree/master/examples)\n[Download and filter by config request](https://github.com/eBay/FeedSDK-Python/blob/master/examples/config_examples.py)\n\n\n---\n## Performance\n|  Category | Type  | Size gz  |  Size unzipped |  Records | Applied Filters | Filter Time | Loading Time | Save Time\n|---|---|---|---|---|---|---|---|---|\n| 11450 | BOOTSTRAP | 4.66 GB | 89.51 GB | 63.2 Million | PriceValue, AvailabilityThresholdType, AvailabilityThreshold | ~ 7 min | ~ 98 min | ~ 2 min\n| 220 | BOOTSTRAP | 867.8 MB | 4.26 GB | 3.3 Million | price, AvailabilityThresholdType, AvailabilityThreshold | ~ 18 sec | ~ 5 min | ~ 37 sec\n| 1281 | BOOTSTRAP | 118.4 MB |  1.06 GB | 812558 | item locations, AcceptedPaymentMethods | ~ 24 sec | ~ 1.2 min | ~ 1.8 min\n| 11232 | BOOTSTRAP | 102.5 MB | 499.9 MB | 405268 | epids, inferredEpids | ~ 0.3 sec | ~ 37 sec | ~ 0.003 sec\n| 550 | BOOTSTRAP | 60.7 MB | 986.5 MB | 1000795 | price, sellers, item locations | ~ 4 sec | ~ 1.4 min | ~ 0.1 sec\n| 260 | BOOTSTRAP | 2.3 MB | 15.6 MB | 24100 | price, AvailabilityThresholdType, AvailabilityThreshold | ~ 0.01 sec | ~ 2 sec | ~ 0.4 sec\n| 220 | DAILY | 13.5 MB | 60.4 MB | 55047 | price, leaf categories, item locations | ~ 0.08 sec | ~ 4 sec | ~ 0.007 sec\n\n\n---\n## Important notes \n\n* Ensure there is enough storage for feed files.\n* Ensure that the file storage directories have appropriate write permissions.\n* In case of failure in downloading due to network issues, the process needs to start again. There is no capability at the moment, to resume.\n\n# License\nCopyright (c) 2018-2019 eBay Inc.\n\nUse of this source code is governed by an Apache 2.0 license that can be found in the LICENSE file or at https://opensource.org/licenses/Apache-2.0.\n',
    'author': 'Lars Erler',
    'author_email': 'lars@xaospage.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/taxaos/FeedSDK-Python',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import logging
from os import path
from utils.file_utils import read_json
from feed.feed_request import Feed
from filter.feed_filter import FeedFilterRequest
from constants.feed_constants import SUCCESS_CODE
from enums.config_enums import ConfigField, FeedField, FilterField
from errors.custom_exceptions import ConfigError
from utils.logging_utils import setup_logging

setup_logging()
logger = logging.getLogger(__name__)


class ConfigRequest(object):
    def __init__(self, feed_obj, filter_request_obj):
        self.feed_obj = feed_obj
        self.filter_request_obj = filter_request_obj

    def __str__(self):
        return '[feed= %s, filter_request= %s]' % (self.feed_obj, self.filter_request_obj)


class ConfigFileRequest(object):
    def __init__(self, config_file_path):
        self.file_path = config_file_path
        self.__token = None
        self.__config_json_obj = None
        self.__requests = []

    @property
    def requests(self):
        return self.__requests

    def parse_requests(self, token=None):
        self.__load_config()
        # populate requests list
        self.__create_requests(token)

    def process_requests(self):
        if not self.requests:
            logger.error('No requests to process')
            return False
        for config_request_obj in self.requests:
            get_response = None
            if config_request_obj.feed_obj:
                feed_req = config_request_obj.feed_obj
                get_response = feed_req.get()
                if get_response.status_code != SUCCESS_CODE:
                    logger.error('Exception in downloading feed. Cannot proceed, continue to the next request\n'
                                 'File Path: %s | Error message: %s\nFeed Request: %s\n', get_response.file_path,
                                 get_response.message, feed_req)
                    continue
            if config_request_obj.filter_request_obj:
                filter_req = config_request_obj.filter_request_obj
                if get_response and get_response.file_path:
                    # override input file path if set
                    filter_req.input_file_path = get_response.file_path
                filter_response = filter_req.filter()
                if filter_response.status_code != SUCCESS_CODE:
                    print(filter_response.message)
        return True

    def __load_config(self):
        # check the path
        if not self.file_path or not path.exists(self.file_path) or path.getsize(self.file_path) == 0:
            raise ConfigError('Config file %s does not exist or is empty' % self.file_path)
        # load the config file
        self.__config_json_obj = read_json(self.file_path)
        # check the config object
        if not self.__config_json_obj:
            raise ConfigError('Could not read config file %s' % self.file_path)

    def __create_requests(self, token):
        if ConfigField.REQUESTS.value not in self.__config_json_obj:
            raise ConfigError('No \"%s\" field exists in the config file %s' % (str(ConfigField.REQUESTS),
                                                                                self.file_path))
        for req in self.__config_json_obj[ConfigField.REQUESTS.value]:
            feed_obj = None
            feed_field = req.get(ConfigField.FEED_REQUEST.value)
            if feed_field:
                feed_obj = Feed(feed_field.get(FeedField.TYPE.value),
                                feed_field.get(FeedField.SCOPE.value),
                                feed_field.get(FeedField.CATEGORY_ID.value),
                                feed_field.get(FeedField.MARKETPLACE_ID.value),
                                token,
                                feed_field.get(FeedField.DATE.value),
                                feed_field.get(FeedField.ENVIRONMENT.value),
                                feed_field.get(FeedField.DOWNLOAD_LOCATION.value),
                                feed_field.get(FeedField.FILE_FORMAT.value))
            filter_request_obj = None
            filter_field = req.get(ConfigField.FILTER_REQUEST.value)
            if filter_field:
                filter_request_obj = FeedFilterRequest(str(filter_field.get(FilterField.INPUT_FILE_PATH.value)),
                                                       filter_field.get(FilterField.ITEM_IDS.value),
                                                       filter_field.get(FilterField.LEAF_CATEGORY_IDS.value),
                                                       filter_field.get(FilterField.SELLER_NAMES.value),
                                                       filter_field.get(FilterField.GTINS.value),
                                                       filter_field.get(FilterField.EPIDS.value),
                                                       filter_field.get(FilterField.PRICE_LOWER_LIMIT.value),
                                                       filter_field.get(FilterField.PRICE_UPPER_LIMIT.value),
                                                       filter_field.get(FilterField.ITEM_LOCATION_COUNTRIES.value),
                                                       filter_field.get(FilterField.INFERRED_EPIDS.value),
                                                       filter_field.get(FilterField.ANY_QUERY.value),
                                                       filter_field.get(FilterField.FILE_FORMAT.value))
            config_request_obj = ConfigRequest(feed_obj, filter_request_obj)
            self.requests.append(config_request_obj)
